<?php // $Id$

/**
 * @file
 * Administration form settings and related functions.
 */

/**
 * Payment settings overview.
 */
function pay_admin_overview() {
  pay_load('pay_form');
  $output = '';

  $hdrs = array(t('Label'), t('Handler'), t('Operations'));
  $info = pay_handlers('pay_method');
  foreach (pay_form::pay_method_list() as $pmid => $pay_method) {
    if (!$pay_method = pay_method_load($pmid)) continue;
    $rows[] = array(
      $pay_method->title(),
      $info[$pay_method->handler()]['title'],
      l(t('edit'), 'admin/settings/pay/'. $pmid .'/edit'),
    );
  }
  if ($rows) {
    $output .= '<h3>' . t('Payment methods') . '</h3>';
    $output .= theme('table', $hdrs, $rows);
  }
  else {
    drupal_set_message(t('You need at least one payment method before you can accept payments on your site. You can add a payment method !add_url.', array('!add_url' => l(t('here'), 'admin/settings/pay/add'))), 'warning');
  }
  return $output;
}

/**
 * List payment forms
 */
function pay_admin_pay_form_list($handler = NULL, $uid = NULL) {

  $hdrs = array(
    array('data' => t('Title'), 'field' => 'Title'),
    array('data' => t('Owner'), 'field' => 'uid'),
    array('data' => t('Handler'), 'field' => 'handler'),
    array('data' => t('Status'), 'field' => 'status'),
    array('data' => t('Total'), 'field' => 'total'),
    array('data' => t('Total paid'), 'field' => 'total_paid'),
    array(),
  );

  $sql = 'SELECT f.*, SUM(t.total) AS total, SUM(t.total_paid) AS total_paid
    FROM {pay_form} f
    LEFT JOIN {pay_transaction} t USING ( pfid ) ';

  if ($handler) {
    $sql .= " WHERE f.handler = '". check_plain($handler) ."'";
    unset($hdrs[2]);
  }

  $sql .= ' GROUP BY f.pfid';

  $res = pager_query($sql . tablesort_sql($hdrs), 20, 0, NULL);
  $path = $_GET['q'];
  $rows = array();
  $pay_forms = array();
  while ($data = db_fetch_object($res)) {
    $pay_form = pay_form_load($data);
    $row = array(
      $pay_form->menu_path() ? l($pay_form->title(), $pay_form->menu_path()) : $pay_form->title(),
      theme('username', $pay_form->user()),
      $pay_form->handler_title(),
      $pay_form->status ? t('active') : t('disabled'),
      round($data->total),
      round($data->total_paid),
      l(t('edit'), $path .'/'. $pay_form->pfid .'/edit', array('query' => 'destination='. $path)),
    );
    if ($handler) unset($row[2]);
    $rows[] = $row;
  }
  if (empty($rows)) $rows[] = array();

  return theme('table', $hdrs, $rows);
}

/**
 * Admin form to create or update payment methods.
 */
function pay_admin_method_form(&$form_state, $pay_method = NULL) {
  if (isset($form_state['storage']['pay_method'])) {
    $pay_method = $form_state['storage']['pay_method'];
  }
  if ($pay_method) {
    $form = array();
    if (!is_object($pay_method)) $pay_method = pay_method_load($pay_method);
    $pay_method->settings_form($form, $form_state);
    $pmid = isset($pay_method->pmid) ? $pay_method->pmid : get_class($pay_method);
    $form['#pay_method'] = $pmid;
  }
  else {
    $form['pay_method'] = array(
      '#type' => 'radios',
      '#title' => t('Payment handler'),
    );
    foreach (pay_handlers('pay_method') as $name => $info) {
      if (!$info['title']) continue; // Exclude unnamed base classes.
      $form['pay_method'][$name] = array(
        '#type' => 'radio',
        '#name' => 'pay_method',
        '#title' => $info['title'],
        '#description' => $info['description'],
        '#return_value' => $name,
      );
    }
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save values'),
  );

  // Vertical tab-ify this form if vertical_tabs module is installed.
  $form['#pre_render'][] = 'vertical_tabs_form_pre_render';
  return $form;
}

/**
 * Submit handler for pay_method_settings.
 */
function pay_admin_method_form_submit($form, &$form_state) {
  if (!$form['#pay_method']) {
    if ($form_state['values']['pay_method']) {
      if (!count($form_state['storage'])) {
        $form_state['storage'] = $form_state['values'];
        $form_state['rebuild'] = TRUE;
        return;
      }
    }
  }
  else {
    $form_state['redirect'] = 'admin/settings/pay';
    unset($form_state['rebuild'], $form_state['storage']);
  }
}

function pay_form_settings(&$form_state, $pay_form, $destination = NULL) {
  if (!is_object($pay_form)) $pay_form = pay_form_load($pay_form);
  $pay_form->settings_form($form, $form_state);

  // Allow destination to be specified in hook_menu().
  if (isset($destination)) {
    $form['destination'] = array(
      '#type' => 'hidden',
      '#value' => $destination,
    );
  }
  return $form;
}
